@@ -139,8 +139,9 @@ using namespace std;
 /* Temporary repositories temporary sources.list */
 #define TEMP_APT_SOURCE_LIST "/var/lib/hildon-application-manager/sources.list.temp"
 
-/* Name of installed applications list file, used for restoring backups */
-#define HILDON_APP_MGR_BACKUP_APPS_FILE "/var/lib/hildon-application-manager/applications.install"
+/* The file where we store our backup data.
+ */
+#define BACKUP_DATA "/var/lib/hildon-application-manager/backup"
 
 
 /* You know what this means.
@@ -606,7 +607,7 @@ void cmd_remove_package ();
 void cmd_clean ();
 void cmd_get_file_details ();
 void cmd_install_file ();
-void cmd_save_applications_install_file ();
+void cmd_save_backup_data ();
 
 /* Commands can request the package cache to be refreshed by calling
    NEED_CACHE_INIT before they return.  The cache will then be
@@ -666,7 +667,7 @@ static char *cmd_names[] = {
   "GET_FILE_DETAILS",
   "INSTALL_FILE",
   "CLEAN",
-  "SAVE_APPLICATIONS_INSTALL_FILE"
+  "SAVE_BACKUP_DATA"
 };
 #endif
 
@@ -752,8 +753,8 @@ handle_request ()
       cmd_install_file ();
       break;
 
-    case APTCMD_SAVE_APPLICATIONS_INSTALL_FILE:
-      cmd_save_applications_install_file ();
+    case APTCMD_SAVE_BACKUP_DATA:
+      cmd_save_backup_data ();
       break;
 
     default:
@@ -3496,149 +3497,70 @@ parse_quoted_word (char **start, char **end, bool term)
   return true;
 }
 
-void
-cmd_save_applications_install_file ()
+static xexp *
+get_backup_catalogues ()
+{
+  xexp *catalogues = xexp_read_file (CATALOGUE_CONF);
+  if (catalogues)
+    {
+      xexp *c = xexp_first (catalogues);
+      while(c)
+	{
+	  xexp *r = xexp_rest (c);
+	  if (xexp_aref_bool (c, "nobackup"))
+	    xexp_del (catalogues, c);
+	  c = r;
+	}
+    }
+
+  return catalogues;
+}
+
+static xexp *
+get_backup_packages ()
 {
-  int res = 0;
-  string name = _config->FindFile("Dir::Etc::sourcelist");
-  FILE *f = fopen (name.c_str(), "r");
   AptWorkerState *state = AptWorkerState::GetCurrent ();
-  gchar **deb_lines_list = NULL;
-  gint n_repositories = 0;
-  GSList *packages_gslist = NULL;
-  gchar **packages_list = NULL;
-  gint n_packages = 0;
-  GSList *node = NULL;
-  gchar *keys_buffer = NULL;
-  GKeyFile *keys = NULL;
-  
-
-  /* read sources.list */
-  if (f)
-    {
-      char *line = NULL;
-      size_t len = 0;
-      ssize_t n;
-      char *start, *end;
-      GSList *deb_lines_gslist = NULL;
-
-      while ((n = getline (&line, &len, f)) != -1)
-	{
-	  if (n > 0 && line[n-1] == '\n')
-	    line[n-1] = '\0';
-	  
-	  if (line == NULL)
-	    break;
-
-	  start = (char *)line;
-	  if (parse_quoted_word (&start, &end, false)
-	      && end - start == 3 && !strncmp (start, "deb", 3))
-	    {
-	      deb_lines_gslist = g_slist_prepend (deb_lines_gslist, g_strdup (line));
-	      n_repositories++;
-	    }
-	}
-
-      /* Transform gslist to array (as required by GKeyFile API */
-      deb_lines_list = g_new0 (char *, n_repositories + 1);
-      n_repositories = 0;
-      for (node = deb_lines_gslist; node != NULL; node = g_slist_next (node))
-	{
-	  deb_lines_list[n_repositories] = (char *) node->data;
-	  n_repositories++;
-	}
-  
-      g_slist_free (deb_lines_gslist);
-      
-      free (line);
-      fclose (f);
-    }
-  else
-    {
-      perror (name.c_str());
-      response.encode_int (0);
-      return;
-    }
 
   if (!ensure_cache ())
-    {
-      g_strfreev (deb_lines_list);
-      response.encode_int (0);
-      return;
-    }
+    return NULL;
+
+  xexp *packages = xexp_list_new ("packages");
 
-  /* Restore installed packages list. Based on cmd_get_package_list */
   pkgDepCache &cache = *(state->cache);
 
   for (pkgCache::PkgIterator pkg = cache.PkgBegin(); !pkg.end (); pkg++)
     {
       pkgCache::VerIterator installed = pkg.CurrentVer ();
 
-      // skip non user packages if requested
+      // skip not-installed packages
+      //
+      if (installed.end ())
+	continue;
+
+      // skip non user packages
       //
       if (!installed.end ()
 	  && !is_user_package (installed))
 	continue;
 
-      // skip not-installed packages if requested
-      //
-      if (installed.end ())
-	continue;
-
-      pkgCache::VerIterator candidate = cache.GetCandidateVer (pkg);
-
-      // skip non user packages if requested
-      //
-      if (!candidate.end ()
-	  && !is_user_package (candidate))
-	continue;
-
-      // skip non-available packages if requested
-      //
-      if (candidate.end ())
-	continue;
-
-      // skip packages that are not installed and not available
-      //
-      if (installed.end () && candidate.end ())
-	continue;
-
-      // Name
-      packages_gslist = g_slist_prepend (packages_gslist, g_strdup(pkg.Name ()));
-      n_packages ++;
+      xexp_cons (packages, xexp_text_new ("pkg", pkg.Name ()));
     }
 
-  if (packages_gslist != NULL)
-    {
-      packages_list = g_new0 (char *, n_packages + 1);
-      n_packages = 0;
-      for (node = packages_gslist; node != NULL; node = g_slist_next (node))
-	{
-	  packages_list[n_packages] = (char *) node->data;
-	  n_packages++;
-	}
-      g_slist_free (packages_gslist);
-    }
-
-  keys = g_key_file_new ();
+  return packages;
+}
 
-  /* Store list of installed package names */
-  g_key_file_set_string_list (keys, "install", "package", packages_list, n_packages);
-  g_key_file_set_string_list (keys, "install", "repo_deb_3", deb_lines_list, n_repositories);
+void
+cmd_save_backup_data ()
+{
+  xexp *catalogues = get_backup_catalogues ();
+  xexp *packages = get_backup_packages ();
 
-  g_strfreev (packages_list);
-  g_strfreev (deb_lines_list);
-  
-  keys_buffer = g_key_file_to_data (keys, NULL, NULL);
-
-  /* Save file */
-  if (keys_buffer != NULL)
+  if (catalogues && packages)
     {
-      res = g_file_set_contents (HILDON_APP_MGR_BACKUP_APPS_FILE, keys_buffer, -1, NULL);
-      g_free (keys_buffer);
+      xexp *data = xexp_list_new ("backup");
+      xexp_append_1 (data, catalogues);
+      xexp_append_1 (data, packages);
+      xexp_write_file (BACKUP_DATA, data);
+      xexp_free (data);
     }
-
-  g_key_file_free (keys);
-
-  response.encode_int (res);
 }
